﻿/*
Copyright (c) 2005-2008, CellAO Team

All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
    * Neither the name of the CellAO Team nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

using System;
using System.Collections.Generic;
using System.IO;
using System.Net;
using System.Text;
using AO.Core;
using AO.Core.Config;

namespace ChatEngine.PacketHandlers
{
    /// <summary>
    /// 
    /// </summary>
    static class Authenticate
    {
        /// <summary>
        /// 
        /// </summary>
        /// <param name="client"></param>
        /// <param name="packet"></param>
        static public void Read(Client client, byte[] packet)
        {
            MemoryStream m_stream = new MemoryStream(packet);
            BinaryReader m_reader = new BinaryReader(m_stream);

            // now we should do password check and then send OK or Error
            // sending OK now

            m_stream.Position = 12;

            short userNameLength = IPAddress.NetworkToHostOrder(m_reader.ReadInt16());
            string userName = Encoding.ASCII.GetString(m_reader.ReadBytes(userNameLength));
            short loginKeyLength = IPAddress.NetworkToHostOrder(m_reader.ReadInt16());
            string loginKey = Encoding.ASCII.GetString(m_reader.ReadBytes(loginKeyLength));

            int characterId = BitConverter.ToInt32(new byte[] { packet[11], packet[10], packet[9], packet[8] }, 0);

            LoginEncryption loginEncryption = new LoginEncryption();

            if (loginEncryption.IsValidLogin(loginKey, client.serverSalt, userName) && loginEncryption.IsCharacterOnAccount(userName, characterId))
            {
                client.Send(new Packets.LoginOK().Create());
            }
            else
            {
                client.Send(new Packets.LoginError().Create());
                client.Server.DisconnectClient(client);
                return;
            }

            // server welcome message
            string m_motd = ConfigReadWrite.Instance.CurrentConfig.Motd;
            // save characters ID in client - note, this is usually 0 if it is a chat client connecting
            client.Character = new Character(characterId, client);

            // add client to connected clients list
            if (client.Server.ConnectedClients.ContainsKey(client.Character.characterId) == false)
            {
                client.Server.ConnectedClients.Add(client.Character.characterId, client);
            }

            // initialize KnownClients list (needed)
            client.KnownClients = new List<int>();
            // add yourself to that list
            client.KnownClients.Add(client.Character.characterId);
            // and give client its own name lookup
            client.Send(Packets.PlayerName.New(client, client.Character.characterId));

            // send server welcome message to client
            client.Send(new Packets.MsgAnonymousVicinity().Create("", m_motd, ""));

            // tell client to join channel "Global"
            // hardcoded right now
            byte[] chn_global = new Packets.ChannelJoin().Create
                (
                new byte[] { 0x04, 0x00, 0x00, 0x23, 0x28 },
                "Global",
                0x8044,
                new byte[] { 0x00, 0x00 }
                );
            client.Send(chn_global);
        }
    }
}
